local astrolabe = DongleStub("Astrolabe-1.0")
local event = {}

local currencies = {
	384, 385, 393, 394, 397, 398, 399, 400, 401,
}
for i=1, #currencies do
	currencies[i] = GetCurrencyInfo(currencies[i])
end

-------------------------
-- Icon object manager --
-------------------------
-- (to avoid creating frames unnecessarily)

local iconPool = {}
iconPool.numIcons = 0
local shownIcons = {}

-- getIcon()
-- Returns an unused icon from the icon pool.
-- If there are no unused icons, create a new icon and return it.
local function getIcon()
	local r
	if #iconPool > 0 then
		-- Remove the icon from the pool, add it to the display, and return it
		r = iconPool[#iconPool]
		iconPool[#iconPool] = nil
		shownIcons[r] = true
		return r
	else
		-- Create a new icon, add it to the display, and return it
		iconPool.numIcons = iconPool.numIcons + 1
		r = CreateFrame("Button", "ExcavatinatorMinimapButton" .. iconPool.numIcons, UIParent, "ExcavatinatorMinimapButtonTemplate")
		shownIcons[r] = true
		return r
	end
end

-- removeIcon(icon)
-- Removes an icon from the minimap, and returns it to the icon pool.
local function removeIcon(icon)
	astrolabe:RemoveIconFromMinimap(icon)
	iconPool[#iconPool + 1] = icon
	shownIcons[r] = nil
end

-- removeAllIcons()
-- Removes all icons from the minimap and returns them to the icon pool
local function removeAllIcons()
	for k, v in pairs(shownIcons) do
		astrolabe:RemoveIconFromMinimap(k)
		iconPool[#iconPool + 1] = k
		shownIcons[k] = nil
	end
end

----------------------
-- Database manager --
----------------------

local db = {}
db.show = true
db.track = true

-- addPosition(optc, optz, optx, opty)
-- Adds a gather position to the database.
-- Positions that are close to each other are considered the same position.
local function addPosition(optc, optz, optx, opty)
	if db.track then
		-- Add or update the position
		local c, z, x, y
		if (optc and optz and optx and opty) then
			c, z, x, y = optc, optz, optx, opty
		else
			c, z, x, y = astrolabe:GetCurrentPlayerPosition()
		end
		if not db[c] then db[c] = {} end -- If no entry for the current continent, create one
		if not db[c][z] then db[c][z] = {} end -- If no entry for the current zone, create one
		local found = 0
		for i=1, #db[c][z] do
			local distance = astrolabe:ComputeDistance(c, z, x, y, c, z, db[c][z][i].x, db[c][z][i].y)
			if distance < 30 then
				-- It's an old position; add one artifact found
				db[c][z][i].gathers = db[c][z][i].gathers + 1
				found = i
				break
			end
		end
		if found == 0 then
			-- It's a new position; add it to the database
			db[c][z][#db[c][z] + 1] = { x = x, y = y, gathers = 1 }
			-- Make an icon for the position
			local icon = getIcon()
			astrolabe:PlaceIconOnMinimap(icon, c, z, x, y)
		end
	end
end

-- updateZone()
-- Hides all icons, then shows all icons for the current zone
local function updateZone()
	removeAllIcons()
	if db.show then
		local c, z, x, y = astrolabe:GetCurrentPlayerPosition()
		if db[c] then
			if db[c][z] then
				for i=1, #db[c][z] do
					local icon = getIcon()
					astrolabe:PlaceIconOnMinimap(icon, c, z, db[c][z][i].x, db[c][z][i].y)
				end
			end
		end
	end
end

---------------------
-- Event functions --
---------------------

function event.ADDON_LOADED(addonName)
	if ExcavatinatorNodeDB then
		db = ExcavatinatorNodeDB
	end
end

function event.PLAYER_LEAVING_WORLD()
	ExcavatinatorNodeDB = db
end

function event.PLAYER_ENTERING_WORLD()
	updateZone()
end

function event.ZONE_CHANGED_NEW_AREA()
	updateZone()
end

function event.CHAT_MSG_CURRENCY(msg)
	for i=1, #currencies do
		if strfind(msg, currencies[i]) then
			addPosition()
		end
	end
end

-------------------
-- Event manager --
-------------------

local function onEvent(s, e, ...)
	if event[e] then
		event[e](...)
	end
end

local eventFrame = CreateFrame("Frame")
eventFrame:SetScript("OnEvent", onEvent)
for k, v in pairs(event) do
	eventFrame:RegisterEvent(k)
end

------------------------------------
-- Communicate with the main file --
------------------------------------

-- Excavatinator:SetMinimapDisplay(isOn)
-- Turns minimap icon display on or off
function Excavatinator:SetMinimapDisplay(isOn)
	db.show = isOn
	updateZone()
end

-- Excavatinator:GetMinimapDisplay()
-- Returns whether minimap display is on or not
function Excavatinator:GetMinimapDisplay()
	return db.show
end

-- Excavatinator:SetMinimapTracking(isOn)
-- Turns dig tracking on or off
function Excavatinator:SetMinimapTracking(isOn)
	db.track = isOn
end

-- Excavatinator:GetMinimapTracking()
-- Returns whether minimap tracking is on or off
function Excavatinator:GetMinimapTracking()
	return db.track
end